/*
 * Streams.java
 *
 * version: 13th December 2007, 19:16
 * authors: yanas
 */
package cz.muni.fi.pb162.streams;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A simple tutorial on input streams.
 * RUN: java cz.muni.fi.pb162.streams.Streams strings.txt
 *
 * @author yanas
 */
public final class Streams {

    /** the encoding used to read the source file */
    private static final String ENCODING = "UTF-8";

    /**
     * This class cannot be instantiated!
     */
    private Streams() {
    }

    /**
     * The entry point of the application.
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // we require exactly one argument
        if (args.length != 1) {
            System.err.println("Invalid number of arguments!");
            System.exit(-1);
        }

        // prepare a variable for an instance of the BufferedReader class
        // this way we can properly close the reader in the finally block
        BufferedReader reader = null;

        try {
            // first create an instance of the FileInputStream class for the given file name
            // then create an instance of the InputStreamReader class that converts raw bytes
            //   into single characters using the given encoding
            // finally create an instance of the BufferedReader allowing us to read strings
            reader = new BufferedReader(
                    new InputStreamReader(new FileInputStream(args[0]), ENCODING));

            // prepare a list for all the strings we are about to read
            // read the first line or null if there are no lines at all
            List<String> strings = new ArrayList<String>();
            String line = reader.readLine();

            // while there are any lines available
            while (line != null) {
                // add the current line to the list and read another one
                strings.add(line);
                line = reader.readLine();
            }

            // sort the list alphabetically
            Collections.sort(strings);

            // print each string per line
            for (String string : strings) {
                System.out.println(string);
            }
        } catch (FileNotFoundException exception) {
            exception.printStackTrace();
        } catch (UnsupportedEncodingException exception) {
            exception.printStackTrace();
        } catch (IOException exception) {
            exception.printStackTrace();
        } finally {
            // close the reader if we managed to open it
            if (reader != null) {
                try {
                    // close() can throw an exception :-/
                    reader.close();
                } catch (IOException exception) {
                    exception.printStackTrace();
                }
            }
        }
    }
}
